import * as React from 'react';
import {augmentImageFile, FileWithDataUrl, getMimeTypeDescription} from '@/utils/blob';
import {formatFileSize, formatNumeral} from '@/utils/numeral';
import clsx from 'clsx';
import {useFileMetadata, useFileUrl, useImageControls} from '@/categories/blob/react';
import {KeyValueTable} from '@/categories/information/react';

type ImageFilePreviewDerivedComponent = HTMLImageElement;

export interface ImageFilePreviewProps extends Omit<React.HTMLProps<ImageFilePreviewDerivedComponent>, 'src' | 'alt'> {
  file?: Partial<FileWithDataUrl>;
  disabled?: boolean;
}

export const ImageFilePreview = React.forwardRef<ImageFilePreviewDerivedComponent, ImageFilePreviewProps>(({
  file,
  className,
  style,
  disabled = false,
  ...etcProps
}, forwardedRef) => {
  const { fileWithUrl, loading: urlLoading } = useFileUrl({ file });
  const { augmentedFile, loading: metadataLoading, error } = useFileMetadata({
    file: fileWithUrl as File,
    augmentFunction: augmentImageFile,
  });
  const {
    fullScreen,
    handleAction,
    imageRef,
    filenameRef,
  } = useImageControls({
    forwardedRef,
  });

  if (!(augmentedFile)) {
    return null;
  }

  const cannotDisplayPicture = Boolean(
    typeof augmentedFile.url !== 'string'
    && error
  );

  return (
    <div
      className={clsx(
        'flex flex-col sm:grid sm:grid-cols-3 gap-8 w-full',
        className,
      )}
      style={style}
    >
      <div className="h-full relative">
        <div className="sm:absolute top-0 left-0 w-full sm:h-full z-[3]">
          {typeof augmentedFile.url === 'string' && (
            <img
              {...etcProps}
              ref={imageRef}
              className={clsx(
                'block h-full max-w-full object-center bg-[#000000]',
                {
                  'object-contain fixed w-full top-0 left-0': fullScreen,
                  'object-cover w-full': !fullScreen,
                },
              )}
              src={augmentedFile.url}
              alt=""
              data-testid="preview"
            />
          )}
          {cannotDisplayPicture && (
            <div className="w-full h-full flex items-center justify-center text-center px-4 bg-[#000000] select-none">
              {error!.message}
            </div>
          )}
        </div>
      </div>
      <div
        className="col-span-2 flex-shrink-0 m-0 flex flex-col gap-4 justify-between"
      >
        <KeyValueTable
          hiddenKeys
          data-testid="infoBox"
          properties={[
            {
              key: 'Name',
              className: 'font-bold',
              valueProps: {
                ref: filenameRef,
                children: augmentedFile.name,
              },
            },
            {
              key: 'Type',
              valueProps: {
                className: clsx(
                  !getMimeTypeDescription(augmentedFile.type, augmentedFile.name) && 'opacity-50'
                ),
                children: getMimeTypeDescription(augmentedFile.type, augmentedFile.name) || '(Loading)',
              },
            },
            {
              key: 'Size',
              valueProps: {
                className: clsx(
                  !formatFileSize(augmentedFile.size) && 'opacity-50'
                ),
                title: `${formatNumeral(augmentedFile.size ?? 0)} bytes`,
                children: formatFileSize(augmentedFile.size) || '(Loading)',
              },
            },
            typeof augmentedFile.metadata?.width === 'number'
            && typeof augmentedFile.metadata?.height === 'number'
            && {
              key: 'Pixel Dimensions',
              valueProps: {
                children: `${formatNumeral(augmentedFile.metadata.width)} × ${formatNumeral(augmentedFile.metadata.height)} pixels`,
              },
            },
            Array.isArray(augmentedFile.metadata?.palette)
            && {
              key: 'Palette',
              valueProps: {
                className: 'mt-1',
                children: augmentedFile.metadata?.palette.map((rgb, i) => (
                  <React.Fragment
                    key={rgb.join(' ')}
                  >
                    {i > 0 && ' '}
                    <span
                      className="whitespace-nowrap inline-block align-top leading-none"
                      title={`rgb(${rgb.join(', ')})`}
                    >
                        <span
                          className="inline-block w-5 h-5 align-middle border border-[#ffffff] ring-1 ring-[#000000]"
                          style={{
                            backgroundColor: `rgb(${rgb.join(' ')})`,
                          }}
                        />
                        <span className="tabular-nums text-xs sr-only">
                          {
                            rgb
                              .map((c) => c
                                .toString()
                                .padStart(4, ' ')
                                .split('')
                                .map((cc, j) => (
                                  <span
                                    key={`${rgb.join(',')}:${c}:${j}`}
                                    className={clsx({
                                      'opacity-0': cc === ' ',
                                    })}
                                  >
                                    {j === 0 && ' '}
                                    {cc === ' ' && j > 0 ? '0' : cc}
                                  </span>
                                ))
                              )
                              .flat()
                          }
                        </span>
                      </span>
                  </React.Fragment>
                )),
              },
            },
          ]}
        />
        <form
          onSubmit={handleAction}
          className="flex gap-4"
        >
          <fieldset
            disabled={disabled || cannotDisplayPicture}
            className="contents"
          >
            <legend className="sr-only">
              Controls
            </legend>
            <button
              type="submit"
              name="action"
              value="toggleFullScreen"
              className={clsx(
                'h-12 flex bg-negative text-primary disabled:text-primary focus:text-secondary active:text-tertiary items-center justify-center leading-none gap-4 select-none',
                'focus:outline-0',
                'disabled:opacity-50 disabled:cursor-not-allowed',
                {
                  'fixed top-0 left-0 w-full h-full opacity-0 z-[3]': fullScreen,
                }
              )}
            >
              <span
                className="block uppercase font-bold h-[1.1em] w-full whitespace-nowrap overflow-hidden text-ellipsis font-semi-expanded"
              >
                Preview
              </span>
            </button>
            {' '}
            <button
              type="submit"
              name="action"
              value="download"
              className={clsx(
                'h-12 flex bg-negative text-primary disabled:text-primary focus:text-secondary active:text-tertiary items-center justify-center leading-none gap-4 select-none',
                'focus:outline-0',
                'disabled:opacity-50 disabled:cursor-not-allowed',
              )}
            >
              <span
                className="block uppercase font-bold h-[1.1em] w-full whitespace-nowrap overflow-hidden text-ellipsis font-semi-expanded"
              >
                Download
              </span>
            </button>
          </fieldset>
        </form>
      </div>
    </div>
  );
});

ImageFilePreview.displayName = 'ImageFilePreview';
